% MTODO 3
% TEST 3: TEST BDS
% VERSIN 2
%
% Las referencias utilizadas han sido, principalmente:
%
%	W.A. Brock, W.D. Dechert, J.A. Scheinkman, B. LeBaron
%	A TEST FOR INDEPENDENCE BASED ON THE CORRELATION DIMENSION
%	Econometric Reviews 15 (1996), 197-235
%
%	L. Kanzler
%	VERY FAST AND CORRECTLY SIZED ESTIMATION OF THE BDS STATISTIC
%	Department of Economics of Oxford University (1999)
%
% Para implementar el algoritmo, los pasos a seguir son:
%
% PASO 1: Introducimos la serie temporal con la que vamos a trabajar.
% PASO 2: Calculamos el estadstico de contraste Wmn.
% PASO 3: Realizamos el contraste de hiptesis.

function t3=test_3_r(serie_temp,mMaxima,parametro_Fcaracteristica,nivel_confianza)

% PASO 1
% INTRODUCIMOS LA SERIE TEMPORAL CON LA QUE VAMOS A TRABAJAR

serie=serie_temp; % serie temporal
n=length(serie); % nmero de elementos de la serie

% PASOS 2 Y 3
% CALCULAMOS EL ESTADSTICO DE CONTRASTE Wmn
% REALIZAMOS EL CONTRASTE DE HIPTESIS

desviacion_tipica=std(serie(:)); % desviacin tpica de los elementos de la serie
epsilon=parametro_Fcaracteristica*desviacion_tipica; %parmetro de distancia

% CALCULAMOS Cn

sumaCn=0;

for cont1=1:n-1
   
    elementoSerie=serie(cont1);
    
    for cont2=cont1+1:n
       
        normaDiferencia=abs(elementoSerie-serie(cont2));
        
        if(normaDiferencia<epsilon)
            
            matriz(cont1,cont2)=1;
            sumaCn=sumaCn+1;
            
        else
            
            matriz(cont1,cont2)=0;
            
        end
        
    end
    
end

Cn=sumaCn/(n*(n-1)/2); % valor de Cn

% CALCULAMOS C1n

sumaAuxiliar=0;

for cont3=1:mMaxima-1
   
    for cont4=cont3+1:n
        
        sumaAuxiliar=sumaAuxiliar+matriz(cont3,cont4);
        
    end
    
    vectorC1n(cont3)=(sumaCn-sumaAuxiliar)/((n-cont3)*(n-cont3-1)/2); % vector de los valores de C1n
    vectorC1n_modificado(cont3)=vectorC1n(cont3)^(cont3+1);
    
end

% CALCULAMOS Kn

sumaKn=0;

for cont5=1:n-2
    
    for cont6=cont5+1:n-1
        
        if(isequal(matriz(cont5,cont6),1)==1)
            
            for cont7=cont6+1:n
                
                sumaKn=sumaKn+matriz(cont6,cont7);
                
            end
            
        end
        
    end
    
end

Kn=sumaKn/(n*(n-1)*(n-2)/6); % valor de Kn

% CALCULAMOS Cmn

nCmn=n;

for m=2:mMaxima
    
    sumaCmn=0;
    
    for cont8=1:nCmn-2
        
        for cont9=cont8+1:nCmn-1
            
            auxiliar=matriz(cont8,cont9)*matriz(cont8+1,cont9+1);
            matriz(cont8,cont9)=auxiliar;
            sumaCmn=sumaCmn+auxiliar;
            
        end
        
    end
    
    vectorCmn(m-1)=sumaCmn/((n-m+1)*(n-m)/2); % vector de los valores de Cmn
    nCmn=nCmn-1;
    
end

% CALCULAMOS Tmn, Vmn y Wmn

for m=2:mMaxima
   
    vectorTmn(m-1)=vectorCmn(m-1)-vectorC1n_modificado(m-1); % vector de los valores de Tmn
    
    suma_Vmn=0;
    
    for cont10=1:m-1
    
        incremento_suma_Vmn=(Kn^(m-cont10))*(Cn^(2*cont10));
        suma_Vmn=suma_Vmn+incremento_suma_Vmn;
    
    end

    vectorVmn(m-1)=2*sqrt(Kn^m+2*suma_Vmn+(m-1)^2*Cn^(2*m)-m^2*Kn*Cn^(2*m-2)); % vector de los valores de Vmn
    vectorWmn(m-1)=sqrt(n-m+1)*vectorTmn(m-1)/vectorVmn(m-1); % vector de los valores de Wmn
    
    if (abs(vectorWmn(m-1))<=norminv((1+nivel_confianza/100)/2,0,1))
    
        vectorTest3(m-1)=1;

    else
    
        vectorTest3(m-1)=0;
    
    end
    
end

t3=vectorTest3;
